import {gql} from '../helpers'
import {   
   actionPromise,
   actionMsgList,
   actionMsgOne,
   actionChatOne
} from '../reducers'
import {   
   actionUploadFile,
} from './mediaActions'
import {   
   actionGetChatById
} from './chatsActions'



export const actionGetMsgsByChat = (chatId, skipCount=0, limitCount=50) => (
   actionPromise('chatMsgs', gql(`query chatMsgs($q: String) {
      MessageFind (query: $q){
         _id
         createdAt
         owner {
            _id
            login
            nick
            avatar {
               url
            }
         }
         text
         chat {
            _id
         }
         media {
            _id
            url
            type
            originalFileName            
         }

         forwardWith {
            _id
         }
         replies {
            _id
         }

         replyTo {
            _id
         }
         forwarded {
            _id
         }
      }     
   }`, { 
         q: JSON.stringify([  { "chat._id": {$in: [chatId]} },
                              { 
                                 sort: [{_id: -1}],  
                                 skip: [skipCount], 
                                 limit: [limitCount] 
                              } 
                           ])
      }
   ))
)

export const actionFullMsgsByChat = (chatId, currentCount, limitCount=50) => (
   async (dispatch, getState) => {
      console.log('MSG STATE', getState())

      console.log(getState().chats[chatId]?.messages?.length, currentCount + limitCount)
      // TODO если длинна массива равна общему количеству то тоже не выполнять
      // +2 так как затягиваем одно сообщение при загрузке всех чатов
      if ( (getState().chats[chatId]?.messages?.length ?? 0) < currentCount + limitCount) {

         let payload = await dispatch(actionGetMsgsByChat(chatId, currentCount, limitCount))
         if (payload) {
            await dispatch(actionMsgList(payload))
         }
      }
   }
)




const actionLastMsgByChat = (chatId) => (
   actionPromise('lastMsg', gql(`query lastMsg($q: String) {
      MessageFind (query: $q){
         _id
         createdAt
         owner {
            _id
            login
            nick
            avatar {
               url
            }
         }
         text
         chat {
            _id
         }
         media {
            _id
            url
            originalFileName
            type
         }

         forwardWith {
            _id
         }
         replies {
            _id
         }

         replyTo {
            _id
         }
         forwarded {
            _id
         }
      }     
   }`, { 
         q: JSON.stringify([  { "chat._id": chatId },
                              { 
                                 sort: [{_id: -1}],
                                 skip: [0], 
                                 limit: [1] 
                              } 
                           ])
      }
   ))
)

export const actionGetAllLastMsg = (chats) => (
   async (dispatch) => {

      let msgReq = chats.map((chat) => dispatch(actionLastMsgByChat(chat._id)))

      for await (const [msg] of msgReq) {
         msg && dispatch(actionMsgOne(msg))
      }
   }
)





export const actionMsgsCount = (chatId) => (
   actionPromise('msgsCount', gql(`query msgsCount($q: String) {
      MessageCount (query: $q)  
   }`, { 
         q: JSON.stringify([ { "chat._id": chatId } ])
      }
   ))
)




export const actionGetMsgById = (msgId) => (
   actionPromise('msgById', gql(`query msgById($q: String) {
      MessageFindOne (query: $q){
         _id
         createdAt
         owner {
            _id
            login
            nick
            avatar {
               url
            }
         }
         text
         chat {
            _id
         }
         media {
            _id
            url
            type
            originalFileName            
         }

         forwardWith {
            _id
         }
         replies {
            _id
         }

         replyTo {
            _id
         }
         forwarded {
            _id
         }
      }     
   }`, { 
         q: JSON.stringify([ { _id: msgId } ])
      }
   ))
)



const actionUpdateMsg = (chatId, text, media, msgId) => (
   actionPromise('updateMsg', gql(`mutation updateMsg($msg: MessageInput) {
      MessageUpsert(message: $msg) {
         _id
         createdAt
         owner {
            _id
            login
            nick
            avatar {
               url
            }
         }
         text
         chat {
            _id
         }
         media {
            _id
            url
            type
            originalFileName            
         }


         forwardWith {
            _id
         }
         replies {
            _id
         }

         replyTo {
            _id
         }
         forwarded {
            _id
         }
      }
   }`, { msg: { _id: msgId, text, chat: {_id: chatId}, media } }
   ))
)


// медиа - массив объектов с ид медиа
export const actionSendMsg = (chatId, text, inputName, files, msgId) => (
   async (dispatch) => {
// тут нужно отделить уже залитые файлы от тех которые лежат локально
// локальные залить и получить ид, с залитых просто получить ид
      const media = []
      for (const file of files) {
         if (file.preview.match(/blob/)) {
            let fileObj = await dispatch(actionUploadFile(inputName, file))
            fileObj && media.push({_id: fileObj?._id})
         } else {
            let fileObj = file
            media.push({_id: fileObj?._id})
         }
      }
      let payload = await dispatch(actionUpdateMsg(chatId, text, media, msgId))
      // if (payload) {
      //    await dispatch(actionMsgOne(payload))
      //    let chatUpdated = await dispatch(actionGetChatById(chatId))
      //    await dispatch(actionChatOne(chatUpdated))
      // }
   }
)


  
   // const actionRemoveMsg = (msgId) => (
   //    actionPromise('removeMsg', gql(`mutation removeMsg($msg: MessageInput) {
   //       MessageDelete(message: $msg) {
   //          _id 
   //       }
   //    }`, { msg: { _id: msgId } }
   //    ))
   // )


