import { createStore, combineReducers, applyMiddleware } from 'redux'
import thunk from 'redux-thunk'
import { promiseReducer } from './promiseReducer'
import { authReducer } from './authReducer'
import { chatsReducer } from './chatsReducer'
import { localStoredReducer } from './localStoredReducer'

import { actionAboutMe } from './findUserActions'
import { 
   actionMsgOne, 
   actionChatOne, 
   actionChatLeft
} from './chatsReducer'

import {   
   actionGetMsgById,
   actionGetChatById, 
   actionFullLogout,
} from '../actions'

import msgSound from '../assets/msgSound.ogg'
import chatSound from '../assets/chatSound.ogg'
// const { io } = require("socket.io-client");


export const store =  createStore (  combineReducers({ 
                                    // promise: localStoredReducer(promiseReducer, 'promise'),
                                    // auth: localStoredReducer(authReducer, 'auth'),
                                    // chats: localStoredReducer(chatsReducer, 'chats'),
                                    promise: promiseReducer, 
                                    auth: authReducer, 
                                    chats: chatsReducer
                                                }),
                                    applyMiddleware(thunk)
                        )

store.dispatch(actionAboutMe())

store.subscribe(() => console.log(store.getState()))


///////////////////////////////////////////////////////////////////

export const socket = window.io("ws://chat.fs.a-level.com.ua")

socket.on('jwt_ok',   (data) => console.log(data))
socket.on('jwt_fail', (error) => {
   console.log(error)
   store.dispatch(actionFullLogout())
})


function playAudio(audio) {
   const newAudio = new Audio(audio)
   newAudio.play()
}


socket.on('msg', async (msg) => { 
   console.log('пришло смс')
   
   const state = store.getState()
   const myId = state.auth.payload?.sub?.id
   const ownerId = msg.owner?._id
   if (myId !== ownerId) {
      playAudio(msgSound)
   }

   const chatId = msg.chat?._id

   await store.dispatch(actionMsgOne(msg)) 

   let msgFull = await store.dispatch(actionGetMsgById(msg._id))
   await store.dispatch(actionMsgOne(msgFull)) 

   let chatUpdated = await store.dispatch(actionGetChatById(chatId))
   await store.dispatch(actionChatOne(chatUpdated))
})

socket.on('chat', async (chat) => { 
   console.log('нас добавили в чат')

   const state = store.getState()
   const myId = state.auth.payload?.sub?.id
   const ownerId = state.chats[chat._id]?.owner?._id

   if (myId !== ownerId) {
      playAudio(chatSound)
   }

   store.dispatch(actionChatOne(chat)) 

   let chatFull = await store.dispatch(actionGetChatById(chat._id))
   await store.dispatch(actionChatOne(chatFull))
})

socket.on('chat_left', async (chat) => { 
   console.log('нас выкинули из чата')

   const state = store.getState()
   const myId = state.auth.payload?.sub?.id
   const ownerId = state.chats[chat._id]?.owner?._id

   if (myId !== ownerId) {
      playAudio(chatSound)
      store.dispatch(actionChatLeft(chat)) 
   }
   store.dispatch(actionChatOne(chat)) 

   let chatFull = await store.dispatch(actionGetChatById(chat._id))
   await store.dispatch(actionChatOne(chatFull))
})




// combineReducers({cart: localStoredReducer(cartReducer, 'cart'),
//                  promise: localStoredReducer(promiseReducer, 'promise') })
//для пользы при работе с промисами надо бы пока PENDING не делать payload undefined 
//при наличии старого payload 
//http://gitlab.a-level.com.ua/gitgod/react-template/src/nosaga/src/reducers/index.js#L13